#!/usr/bin/perl
#$Id: que-server.pl,v 1.5 2023/10/06 14:22:49 sqko Exp $
use POSIX qw(strftime);
use Fcntl qw(:DEFAULT :flock);
use IO::Socket;
use IO::Interface::Simple;
use strict;
use warnings;
use Sys::Hostname;

our $debug = 0;

##### Setup hash of functions ####
my %cmdlst = ();
$cmdlst{'quit'} = \&quit;
$cmdlst{'debug'} = \&set_debug;
$cmdlst{'impedance'} = \&add;
$cmdlst{'chrono'} = \&add;
$cmdlst{'potsweep'} = \&add;
$cmdlst{'get_item'} = \&shift_que;
$cmdlst{'list'} = \&print_list;
$cmdlst{'del_item'} = \&del_item;
$cmdlst{'add_item'} = \&add_simple;

my @cue = ();
my $cuecount = 0;

my $port = 4041;
#### Try to get ip address of netwrok interface ###
my @ilst = ('eno','eth','em','en','lo');
my $local = '';
my @ifl = IO::Interface::Simple->interfaces();
OUTER:
foreach my $try (@ilst)
{
    foreach my $interface (@ifl)
    {
	if ($interface =~ /$try/)
	{
	    my $if = IO::Interface::Simple->new($interface);
	    $local = $if->address();
	    last OUTER if ($local);
	}
    }
}
### Use localhost if everything else fails ###
$local = 'localhost' unless ($local);

$| = 1;

### Bind to socket ###
my $sock = new IO::Socket::INET(
    LocalHost => $local,
    LocalPort => $port,
    Proto  => 'tcp',
    Listen => 10,
    Reuse  => 1
);
die "Could not create socket: $!\n" unless $sock;
$| = 1;
my $new_sock;
while ( $new_sock = $sock->accept() ) 
{
    if ($debug)
    {
	print strftime "%Y:%m:%d:%H:%M:%S", localtime;
    }
    my $other_end = getpeername($new_sock);
    my ( $port, $iaddr ) = unpack_sockaddr_in($other_end);
    my $ip_address = inet_ntoa($iaddr);
    print "  ",$ip_address, "\n" if ($debug);
    if (my $str = <$new_sock>) {
        my ($cmd,@args) = split (/[\s\t]+/,$str);
	$cmd = lc($cmd);
	print $str if ($debug);
	if ($cmd && $cmdlst{$cmd})
	{
	    my $res = $cmdlst{$cmd}->($cmd,@args);
	    print "Result: $res\n\n" if ($debug && defined($res));
	    print $new_sock $res if (defined($res));
	}
	close $new_sock;
    }
    else
    {
	close $new_sock;
    }
}
close($sock);
exit;


sub add()
{
    ### Expected string:  impedance user mode setup riguser ip
    unless (@_ > 5)
    {
	return "Not enough arguments to determine correct return path for command!\n";
    }
    my %item = ();
    $item{'time'} = localtime();
    $item{'id'} = ++$cuecount;
    $item{'ip'} = pop;
    $item{'riguser'} = pop;
    $item{'cmd'} = join(' ',@_);
    push @cue, \%item;
    return $cuecount.' '.scalar(@cue);
}


sub add_simple()
{
    ### Expected string:  add_item cmd [args] riguser ip
    unless (@_ > 3)
    {
	return "Not enough arguments to determine correct return path for command!\n";
    }
    my %item = ();
    shift;  ## Remove the 'add_item' argument ## 
    $item{'time'} = localtime();
    $item{'id'} = ++$cuecount;
    $item{'ip'} = pop;
    $item{'riguser'} = pop;
    $item{'cmd'} = join(' ',@_);
    push @cue, \%item;
    return $cuecount.' '.scalar(@cue);
}

sub print_list()
{
    my $string;
    foreach (@cue)
    {
	$string .= join(' ',$$_{'id'},$$_{'ip'},$$_{'riguser'},$$_{'cmd'},$$_{'time'})."\n";
    }
    return undef unless ($string);
    chomp $string;
    return $string;
}


sub shift_que()
{
    return undef unless (@cue);
    my $item = shift(@cue);
    return join(' ',$$item{'id'},$$item{'ip'},$$item{'riguser'},$$item{'cmd'});
}

sub del_item()
{
    my $id = $_[1];
    return undef unless ($id);
    my @lcue = ();
    my $str = "Could not delete Item with id $id, Item not found!";
    foreach (@cue)
    {
	if ($$_{'id'} == $id)
	{
	    $str = 'Item with id '.$id.' succesfully deleted from que!';
	}
	else
	{
	    push @lcue,$_;
	}
    }
    @cue = @lcue;
    return $str;
}



sub quit
{
    print $new_sock "quit command received, exiting", "\n\n";
    close $new_sock;
    close $sock;
    exit;
}

sub set_debug
{
    shift;
    my $cmd = shift;
    if ($cmd)
    { 
	if ($cmd =~ /enable/i)
	{
	    $debug = 1;
	    return "Debug on";
	}
	if ($cmd =~ /disable/i)
	{
	    $debug = 0;
	    return "Debug off";
	}
    }
    else
    {
	if ($debug)
	{
	    $debug = 0;
	    return "Debug off";
	}
	$debug = 1;
	return "Debug on";
    }
}


