#!/usr/bin/perl
#$Id: que_control.pl,v 1.14 2023/11/21 08:57:47 sqko Exp $
use strict;
use warnings;
use Socket;
use Fcntl qw/:DEFAULT :flock/;
use IO::Interface::Simple;
use Sys::Hostname;
use Elchemea qw(get_config_value CGI_client);
use SocketClient qw(GPIB_client socket_client);
use SOAP::Lite;
my $debug = 0;
if (scalar(@ARGV) > 0)
{
    for(my $x = 0; $x < scalar(@ARGV);$x++)
    {
        if ($ARGV[$x] =~ /--debug/)
        {
	    $debug = 1;
        }
    } 
}
our @SOAP_ERRORS = ();
my $hostname = hostname;
my $wsdl = "http://$hostname/wsdl/timeslot.wsdl";
my $local_soap = SOAP::Lite->on_fault(\&soapGetBad)->service($wsdl);

my %soap = ();
my $conf = '/home/elchemea/multiplex.conf';
## Check for already running control program
my $lf = '/home/elchemea/que_control.lock';
sysopen(DBL, $lf, O_WRONLY | O_CREAT) or die "Could not open lockfile $lf $!!\n";
my $is_locked = flock( DBL, LOCK_EX | LOCK_NB );
if ( $is_locked == 0 ) {
    exit;
}
my $this_host = undef;
my @ilst = ('eth','em','en','lo');
my @ifl = IO::Interface::Simple->interfaces();
 OUTER:
foreach my $try (@ilst)
{
    foreach my $interface (@ifl)
    {
	if ($interface =~ /$try/)
	{
	    my $if = IO::Interface::Simple->new($interface);
	    $this_host = $if->address();
	    last OUTER if ($this_host);
	}
   }
}
chomp $this_host;
my $port  = &get_config_value('remote','address-remote');
$port = 4040 unless ($port);
$this_host .= ':'.$port;

### Do not start immediately ###
sleep(10);
while (1)
{
    sleep(10);
    my $item = &que_client('get_item');
    next unless ($item);
    chomp $item;
    print $item,"\n" if ($debug);
    my @l = split(/\s+/,$item);
    my $ip = $l[1];
    my $rig = $l[2];
    my $true_rig = $rig;
    if ($true_rig && $true_rig =~ /([a-zA-Z\d]+)/)
    {
        $true_rig = $1;
    }
    my $c = &get_channel_rig($rig);
    print "Setting multiplex for ",$c,"\n" if ($debug);
    foreach ('1,1','1,2','2,1','2,2','3,1')
    {
	my $cmd = "CHAN $_,$c";
	print $cmd,"\n" if ($debug);
	print GPIB_client('w',10,$cmd);	
    }
    sleep(10);
    if ($l[3] =~ /timeslot/)
    {
	&celltest_client($ip,$true_rig,'timeslot_begin');
	sleep($l[4]);
	&celltest_client($ip,$true_rig,'timeslot_end');
	sleep(70);
	my $testfile = &get_config_value('measure','controlfile');
	unlink($testfile) if (-e $testfile); ### Elchmea v. 5.x ###
	my $measurelock = '/tmp/measurelock.lock';
	unlink($measurelock) if (-e $measurelock);  ### Elchemea v. 6.x ###
	sleep(10);
	print "Resetting multiplex\n" if ($debug);
	GPIB_client('w',10,'RESET');
	next;
    }
    my $user = $l[4];
    my $mode = $l[5];
    my $res = CGI_client($l[3],$user,$l[6],$mode);
    if ($res && ($res =~ /Impedance/ || $res =~ /Chrono/ || $res =~ /Potsweep/))
    {
	&celltest_client($ip,$true_rig,'cmdlog',"Impedance started: $res");
	$res =~ /Session\s+(\d+).*File\s+(\d+).*time\s+([\d\.]+)/;
	my $session = $1+0;
	my $fileid = $2+0;
	my $wait = $3+30;
	sleep(60);
	my $resfile = CGI_client('get_file',$user,$mode,$session,$fileid);
	unless ($resfile && $resfile =~ /MEASURING/)
	{
	    $res = CGI_client($l[3],$user,$mode,$l[6]);
	    unless ($res && ($res =~ /Impedance/ || $res =~ /Chrono/ || $res =~ /Potsweep/))
	    {
		&celltest_client($ip,$true_rig,'cmdlog',"WARNING: Impedance could not be started, system not responding");
		GPIB_client('w',10,'RESET');
		next;		
	    }
	    sleep(60);
	    $res =~ /Session\s+(\d+).*File\s+(\d+).*time\s+([\d\.]+)/;
	    $session = $1+0;
	    $fileid = $2+0;
	    $wait = $3+30;
	    sleep(60);
	    $resfile = CGI_client('get_file',$user,$mode,$session,$fileid);
	    unless ($resfile && $resfile =~ /MEASURING/)
	    {
		&celltest_client($ip,$true_rig,'cmdlog',"WARNING: Impedance could not be started, tried 2 times without success!");
		GPIB_client('w',10,'RESET');
		next;
	    }
	}
	sleep($wait-60);
	$resfile = CGI_client('get_file',$user,$mode,$session,$fileid);
	while ($resfile && $resfile =~ /^MEASURING/ && $wait > 0)
	{
	    $wait = $wait - 30;
	    sleep(30);
	    $resfile = CGI_client('get_file',$user,$mode,$session,$fileid);
	}
	if ($resfile && length($resfile) > 100)
	{
	    &celltest_client($ip,$true_rig,'impedance_ok',$this_host,$user,$mode,$session,$fileid);
	}
	else
	{
	    &celltest_client($ip,$true_rig,'cmdlog',"WARNING: Impedance data could not be aquired, system not responding");
	}
    }
    else
    {
	&celltest_client($ip,$true_rig,'cmdlog',"WARNING: Impedance could not be started, system not responding!");
    }
    GPIB_client('w',10,'RESET');
}
exit;


sub get_channel_rig()
{
    my $rig = shift;
    my %cf = ();
    open MYIN,'<',$conf or die "Could not open file $conf $!";
    while (<MYIN>)
    {
	if (/(\w+)[\t\s]+(\d+)/)
	{
	    $cf{$1} = $2;
	}
    }
    close MYIN;
    return $cf{$rig} if (exists($cf{$rig}));
    return undef;
}


sub que_client(@)
{
    my $res = undef;
    eval {
        my @args = @_;
        my $cmd = shift(@args);
        my $res = $local_soap->$cmd(@args);
    };
    if ($res)
    {
        print scalar(localtime),": Que client result:$res\n" if ($debug);
        return $res;
    }
    ## IF SOAP fails try socket
    my ($remote,$port);
    #### Try to get ip address of netwrok interface ###
    my @ilst = ('eth','em','en','lo');
    $remote = '';
    my @ifl = IO::Interface::Simple->interfaces();
  OUTER:
    foreach my $try (@ilst)
    {
	foreach my $interface (@ifl)
	{
	    if ($interface =~ /$try/)
	    {
		my $if = IO::Interface::Simple->new($interface);
		$remote = $if->address();
		last OUTER if ($remote);
	    }
	}
    }
    ### Use localhost if everything else fails ###
    $remote = 'localhost' unless ($remote);
    $port    =  4041;
    $res = socket_client($remote,$port,@_);
    print scalar(localtime),": Que client result:$res\n" if ($debug);
    return $res;
}

sub celltest_client($$@)
{
    my @arg = @_;
    my $remote = shift;
    my $rig = shift;
    $rig =~ /(\d+)/;
    $rig = $1;
    if ($remote =~ /^[\d\.]+$/)
    {
	$remote = get_hostname($remote);
    }
    my $res = undef;
    eval {
	unless ($soap{$remote})
	{
	    my $wsdl = 'https://'.$remote.'/wsdl/cgi_service.wsdl';
	    $soap{$remote} = SOAP::Lite->service($wsdl);
	}
	print $soap{$remote}->ping(),"\n" if ($debug);
	my $cmd = shift;
	print scalar(localtime),': ',join("\t",$remote,$cmd,$rig,@_),"\n" if ($debug);
	$res = $soap{$remote}->$cmd($rig,@_);
    };
    if ($@)
    {
	warn scalar(localtime),': ',$@,"\n";
	$res = &celltest_client_old(@arg)
    }
    print scalar(localtime),": Result: $res\n";
    return $res;
}

sub celltest_client_old($$@)
{
    my $remote = shift;
    my $rig = shift;
    if ($rig =~ /(\d+)/)
    {
	$rig = $1;
    }
    else
    {
	warn("Rig $rig not valid!");
	return undef;
    }
    my $port = 2000+$rig;
    return socket_client($remote,$port,@_);
}


sub get_hostname($)
{
    my $ip = shift;
    if ($ip && $ip =~ /([\d\.]+)/)
    {
	my @res = `/bin/host $1`;
	foreach my $host (@res)
	{
	    ### Ugly Ugly Ugly hack for FUBAR DNS setup at DTU where machines get a win.dtu.dk dns record in addition to the requeste one %&##&
	    next if ($host =~ /win\.dtu\.dk/);
	    if ($host =~ /([\-\w\.]+)[\s\t]*$/)
	    {
		return $1;
	    }
	}

    }
    return $ip;
}

sub soapGetBad {
    my $soap = shift;
    my $res = shift;
    if( ref( $res ) ) {
        chomp( my $err = $res->faultstring );
        push( @SOAP_ERRORS, "SOAP FAULT: $err" );
    }
    else {
        chomp( my $err = $soap->transport->status );
        push( @SOAP_ERRORS, "TRANSPORT ERROR: $err" );
    }
    warn join(@SOAP_ERRORS),"\n";
    return new SOAP::SOM;
}

