#!/usr/bin/perl
#$Id: run_multiimp.pl,v 1.7 2019/02/08 09:22:02 sqko Exp $
use strict;
use warnings;
use AFM;
use SOAP::Lite;
use IO::Interface::Simple;
our @SOAP_ERRORS = ();
if (@ARGV < 2)
{
    warn "Usage: $0 rignumber username\n";
    exit 1;
}
my $rig = shift;
my $user = shift;
&check_run($rig);
my $soap;
my $impsoap;

my $host = &get_cv('impedance','multiplexer_host');
my $port = &get_cv('impedance','multiplexer_port');
my $qp = &get_cv('impedance','multiplexer_que_port');
my $mode = &imp_soap($host.':'.$port,'mode');
chomp $mode;

my $session = &imp_soap($host.':'.$port,'session',$user);
chomp $session;
exit 1 unless ($session);

my @ilst = ('em','eth');
my $local = '';
my @ifl = IO::Interface::Simple->interfaces();
OUTER:
foreach my $try (@ilst)
{
    foreach my $interface (@ifl)
    {
        if ($interface =~ /$try/)
        {
            my $if = IO::Interface::Simple->new($interface);
            $local = $if->address();
            last OUTER if ($local);
        }
    }
}
my $res = &remote_soap($host.':'.$port,'impedance',$user,$mode,$session,'rig'.$rig);
print $res;


sub remote_soap
{
    my $addr = shift;
    my $cmd = shift;
    my @args = @_;
    my $res;
    eval {
        my $saddr = $addr;
        $saddr =~  s/\:.*$//;
        my $wsdl = 'https://'.$saddr.'/wsdl/timeslot.wsdl';
        $soap = SOAP::Lite->on_fault(\&soapGetBad)->service($wsdl) unless ($soap);
        $res = $soap->$cmd(@args);
    };
    if ($@)
    {
	print $@,"\n";
        ## Fallback ##
        my $command = &get_cv('impedance','remote_client')." $addr $cmd ".join(' ',@args);
        $res = `$command`;
    }
    return $res;
}

sub imp_soap
{
    my $addr = shift;
    my $cmd = shift;
    my @args = @_;
    my $res;
    eval {
        my $saddr = $addr;
        $saddr =~  s/\:.*$//;
        my $wsdl = 'https://'.$saddr.'/wsdl/elchemea.wsdl';
        $impsoap = SOAP::Lite->on_fault(\&soapGetBad)->service($wsdl) unless ($impsoap);
        $res = $impsoap->$cmd(@args);
    };
    if ($@)
    {
	print $@,"\n";
        ## Fallback ##
        my $command = &get_cv('impedance','remote_client')." $addr $cmd ".join(' ',@args);
        $res = `$command`;
    }
    return $res;
}


sub soapGetBad {
    my $soap = shift;
    my $res = shift;
    if( ref( $res ) ) {
        chomp( my $err = $res->faultstring );
        push( @SOAP_ERRORS, "SOAP FAULT: $err" );
    }
    else {
        chomp( my $err = $soap->transport->status );
        push( @SOAP_ERRORS, "TRANSPORT ERROR: $err" );
    }
    warn join(@SOAP_ERRORS),"\n";
    return new SOAP::SOM;
}
