#!/usr/bin/perl -w
#$Id: timeslot.cgi,v 1.7 2018/10/11 15:21:52 sqko Exp $
use strict;
use warnings;
use Data::Dumper;
use SOAP::Transport::HTTP;

my $server = SOAP::Transport::HTTP::CGI
    -> dispatch_to('Timeslot')
    -> handle;

package Timeslot;
use SocketClient;
use vars qw(@ISA);
@ISA = qw(SOAP::Server::Parameters);
use Sys::Hostname;

=begin WSDL
_DOC Returns the hostname of the server
_RETURN $string
=end WSDL

=cut

sub ping()
{
    my $class = shift;
    return SOAP::Data->new(name=>'result',value=>hostname(),type=>'string');
}

=begin WSDL
_DOC Sets debug level
_IN Enable $string Optional (enable/disable), if not specified, toggel debug
_RETURN $string
=end WSDL

=cut

sub debug
{
    my $class = shift;
    my $enable = shift;
    my $str = undef;
    eval {
	my $res = SocketClient::socket_client_raw(hostname(),4041,'debug',$enable);
	my $str =SOAP::Data->new(name=>'result',value=>$res,type=>'string');
    };
    if ($@)
    {
        return SOAP::Fault->faultcode('Server error')
            ->faultstring($@)
            ->faultdetail(bless {code => 1} => 'BadError')
            ->faultactor('https://'.hostname().'/Timeslot');	
    }
    return $str;
}


=begin WSDL
_DOC Adds an item to the que. Only to be used with RFCcontrol version 6.x timeslot commands
_IN command $string Command to add
_IN Time $string Duration in minutes of command
_IN rig $string RFCcontrol rig name (ex. rig14) requesting the measurement
_RETURN $string
=end WSDL

=cut


sub add_item
{
    my $class = shift;
    my $cmd = shift;
    my $time = shift;
    my $rig = shift;
    my $ip = $ENV{REMOTE_ADDR};
    my @args = ('add_item',$cmd,$time,$rig,$ip);
    my $str = undef;
    eval {
	my $res = SocketClient::socket_client_raw(hostname(),4041,@args);
	$str =  SOAP::Data->new(name=>'result',value=>$res,type=>'string');
    };
    if ($@)
    {
        return SOAP::Fault->faultcode('Server error')
            ->faultstring($@)
            ->faultdetail(bless {code => 1} => 'BadError')
            ->faultactor('https://'.hostname().'/Timeslot');	
    }
    return $str;
}



=begin WSDL
_DOC Adds a timeslot request to the que. Can only be used with RFCcontrol version 6.x
_IN Time $string Duration in minutes of the timeslot
_IN rig $string RFCcontrol rig name (ex. rig14) requesting the timeslot
_RETURN $string
=end WSDL

=cut


sub timeslot
{
    my $class = shift;
    my $time = shift;
    my $rig = shift;
    my $ip = $ENV{REMOTE_ADDR};
    my @args = ('add_item','timeslot',$time,$rig,$ip);
    my $str = undef;
    eval {
	my $res = SocketClient::socket_client_raw(hostname(),4041,@args);
	$str =  SOAP::Data->new(name=>'result',value=>$res,type=>'string');
    };
    if ($@)
    {
        return SOAP::Fault->faultcode('Server error')
            ->faultstring($@)
            ->faultdetail(bless {code => 1} => 'BadError')
            ->faultactor('https://'.hostname().'/Timeslot');	
    }
    return $str;
}


=begin WSDL
_DOC Adds an impedance scan to the que
_IN user $string Local Elchemea username to use for measurement
_IN mode $string Mode of impedance system (not used but nescesarry for backwards compatibility)
_IN session $int Session configuration to use for measurement
_IN rig $string RFCcontrol rig name (ex. rig14) requesting the measurement
_RETURN $string
=end WSDL

=cut


sub impedance
{
    my $class = shift;
    my $user = shift;
    my $mode = shift;
    my $session = shift;
    my $rig = shift;
    my $ip = $ENV{REMOTE_ADDR};
    my @args = ('impedance',$user,$mode,$session,$rig,$ip);
    my $str = undef;
    eval {
	my $res = SocketClient::socket_client_raw(hostname(),4041,@args);
	$str =  SOAP::Data->new(name=>'result',value=>$res,type=>'string');
    };
    if ($@)
    {
        return SOAP::Fault->faultcode('Server error')
            ->faultstring($@)
            ->faultdetail(bless {code => 1} => 'BadError')
            ->faultactor('https://'.hostname().'/Timeslot');	
    }
    return $str;
}


=begin WSDL
_DOC Adds an impedance scan to the que
_IN user $string Local Elchemea username to use for measurement
_IN mode $string Mode of impedance system (not used but nescesarry for backwards compatibility)
_IN session $int Session configuration to use for measurement
_IN rig $string RFCcontrol rig name (ex. rig14) requesting the measurement
_RETURN $string
=end WSDL

=cut


sub chrono
{
    my $class = shift;
    my $user = shift;
    my $mode = shift;
    my $session = shift;
    my $rig = shift;
    my $ip = $ENV{REMOTE_ADDR};
    my @args = ('chrono',$user,$mode,$session,$rig,$ip);
    my $str = undef;
    eval {
	my $res = SocketClient::socket_client_raw(hostname(),4041,@args);
	$str =  SOAP::Data->new(name=>'result',value=>$res,type=>'string');
    };
    if ($@)
    {
        return SOAP::Fault->faultcode('Server error')
            ->faultstring($@)
            ->faultdetail(bless {code => 1} => 'BadError')
            ->faultactor('https://'.hostname().'/Timeslot');	
    }
    return $str;
}


=begin WSDL
_DOC Adds an impedance scan to the que
_IN user $string Local Elchemea username to use for measurement
_IN mode $string Mode of impedance system (not used but nescesarry for backwards compatibility)
_IN session $int Session configuration to use for measurement
_IN rig $string RFCcontrol rig name (ex. rig14) requesting the measurement
_RETURN $string
=end WSDL

=cut


sub potsweep
{
    my $class = shift;
    my $user = shift;
    my $mode = shift;
    my $session = shift;
    my $rig = shift;
    my $ip = $ENV{REMOTE_ADDR};
    my @args = ('potsweep',$user,$mode,$session,$rig,$ip);
    my $str = undef;
    eval {
	my $res = SocketClient::socket_client_raw(hostname(),4041,@args);
	$str = SOAP::Data->new(name=>'result',value=>$res,type=>'string');
    };
    if ($@)
    {
        return SOAP::Fault->faultcode('Server error')
            ->faultstring($@)
            ->faultdetail(bless {code => 1} => 'BadError')
            ->faultactor('https://'.hostname().'/Timeslot');	
    }
    return $str;
}


=begin WSDL
_DOC List the content of the que
_RETURN @string
=end WSDL

=cut


sub list
{
    my $class = shift;
    my @list = ();
    eval {
	my $res = SocketClient::socket_client_raw(hostname(),4041,'list');
	foreach my $str (split(/\n/,$res))
	{
	    my $d = SOAP::Data->new(name=>'item',type=>'string',value=>$str);
	    push @list,$d;
	}
    };
    if ($@)
    {
        return SOAP::Fault->faultcode('Server error')
            ->faultstring($@)
            ->faultdetail(bless {code => 1} => 'BadError')
            ->faultactor('https://'.hostname().'/Timeslot');	
    }
    return @list;
}


=begin WSDL
_DOC Removes an item from the que
_IN ID $int ID number to delete from que
_RETURN $string
=end WSDL

=cut

sub del_item
{
    my $class = shift;
    my $id = shift;
    my $str = undef;
    eval {
	my $res = SocketClient::socket_client_raw(hostname(),4041,'del_item',$id);
	$str =  SOAP::Data->new(name=>'result',value=>$res,type=>'string');
    };
    if ($@)
    {
        return SOAP::Fault->faultcode('Server error')
            ->faultstring($@)
            ->faultdetail(bless {code => 1} => 'BadError')
            ->faultactor('https://'.hostname().'/Timeslot');	
    }    
    return $str;
}
